#include <limits.h>

/*@ predicate sorted{L}(long *t, integer a, integer b) =
  @  \forall integer i,j; a <= i <= j <= b ==> t[i] <= t[j];
  @*/

/*@ requires 0 <= n <= (INT_MAX / 2) && \valid(t + (0 .. n - 1));
  @ requires 0 <= l <= n;
  @ requires l - 1 <= u < n;
  @ ensures \result == -1 || l <= \result <= u;
  @ ensures \forall integer k; 0 <= k < n ==> t[k] == \old(t[k]);
  @ assigns \nothing;
  @ behavior success:
  @   ensures (\result >= 0) ==> t[\result] == v;
  @ behavior failure:
  @   assumes sorted(t, l, u);
  @   ensures (\result == -1) ==> \forall integer k; l <= k <= u ==> t[k] != v;
  @*/
int binary_search_between(long t[], int n, long v, int l, int u) {
  if (l <= u) {
    int m = (l + u) / 2;
    //@ assert l <= m <= u;
    if (t[m] < v) {
      return binary_search_between(t, n, v, m + 1, u);
    } else if (t[m] > v) {
      return binary_search_between(t, n, v, l, m - 1);
    } else {
      return m;
    }
  } else
    return -1;
}

/*@ requires 0 <= n <= (INT_MAX / 2) && \valid(t + (0 .. n - 1));
  @ ensures -1 <= \result < n;
  @ assigns \nothing;
  @ behavior success:
  @   ensures \result >= 0 ==> t[\result] == v;
  @ behavior failure:
  @   assumes sorted(t, 0, n - 1);
  @   ensures \result == -1 ==> \forall integer k; 0 <= k < n ==> t[k] != v;
  @*/
int binary_search(long t[], int n, long v) {
  return binary_search_between(t, n, v, 0, n - 1);
}
