#include <stdint.h>

const int32_t P = 7681;
const int32_t ZETA = 3777;
const int32_t ZETA_PINV = 28865;

/*@
  @ requires \valid(r1) && \valid(r2);
  @ requires \separated(r1, r2, &P, &ZETA, &ZETA_PINV);
  @ requires (-4096 < f000 < 4096);
  @ requires (-4096 < f128 < 4096);
  @ assigns *r1, *r2;
  @*/
void foo(const int32_t f000, const int32_t f128, int32_t *r1, int32_t *r2) {
  int32_t f128_mul_zeta_pinv_low = (ZETA_PINV * f128) % (1 << 16);
  int32_t f128_mul_zeta = ZETA * f128;
  int32_t f128_mul_zeta_high = f128_mul_zeta >> 16;
  int32_t f128_mul_zeta_pinv_low_mul_p = f128_mul_zeta_pinv_low * P;
  //@ assert f128_mul_zeta_pinv_low_mul_p % (1 << 16) == f128_mul_zeta % (1 << 16);
  int32_t f128_mul_zeta_pinv_low_mul_p_high = f128_mul_zeta_pinv_low_mul_p >> 16;
  *r1 = f000 + f128_mul_zeta_high - f128_mul_zeta_pinv_low_mul_p_high;
  *r2 = f000 - f128_mul_zeta_high + f128_mul_zeta_pinv_low_mul_p_high;
}

int main(void) {
  int32_t f000 = 123, f128 = 567, r1, r2;
  foo(f000, f128, &r1, &r2);
  return 0;
}
