
/*@ requires \valid(a) && \valid(b);
  @ ensures *a == \old(*b) && *b == \old(*a);
  @ assigns *a, *b;
  @*/
void swap(int *a, int *b)
{
  int tmp = *a;
  *a = *b;
  *b = tmp;
}

/*@
  @ requires \valid(a) && \valid(b) && \valid(c) && \separated(a, b, c);
  @ ensures *a <= *b <= *c;
  @ ensures { *a, *b, *c } == { \old(*a), \old(*b), \old(*c) };
  @ ensures \forall int* x, int* y;
              \subset({ x, y }, { a, b, c }) && \separated(x, y) && \old(*x) == \old(*y) ==>
                \exists int* u, int* v;
                  \subset({ u, v }, { a, b, c }) && \separated(u, v) && *u == *v == \old(*x);
  @ assigns *a, *b, *c;
  @*/
void order3(int *a, int *b, int *c) {
  if (*a > *b) swap(a, b);
  if (*a > *c) swap(a, c);
  if (*b > *c) swap(b, c);
}

void test() {
  int a1 = 5, b1 = 3, c1 = 4;

  order3(&a1, &b1, &c1);
  //@ assert a1 == 3 && b1 == 4 && c1 == 5;

  int a2 = 2, b2 = 2, c2 = 2;
  order3(&a2, &b2, &c2);
  //@ assert a2 == 2 && b2 == 2 && c2 == 2;

  int a3 = 4, b3 = 3, c3 = 4;
  order3(&a3, &b3, &c3);
  //@ assert a3 == 3 && b3 == 4 && c3 == 4;

  int a4 = 4, b4 = 5, c4 = 4;
  order3(&a4, &b4, &c4);
  //@ assert a4 == 4 && b4 == 4 && c4 == 5;
}
